package Plugins::DRS::Plugin;

use strict;

use vars qw($VERSION);
use HTML::Entities;
use Digest::SHA1;
use HTTP::Request;

use Slim::Networking::SimpleAsyncHTTP;
use Slim::Networking::SqueezeNetwork;
use Slim::Utils::Log;
use Slim::Utils::Prefs;

use constant SEARCH_URL => 'http://www.herger.net/cgi-bin/getartwork.cgi';

my $log = Slim::Utils::Log->addLogCategory({
	'category'     => 'plugin.drs',
	'description'  => 'PLUGIN_DRS_META',
});

my $prefs = preferences('server');

use constant cacheTTL => 15;

my $urls = {
	drs1 => 'http://www.drs.ch/import/webradio/playerinfos_infoszursendung_drs1.html',
	drs2 => 'http://www.drs.ch/import/webradio/playerinfos_infoszursendung_drs2.html',
	drs3 => 'http://www.drs.ch/import/webradio/playerinfos_infoszursendung_drs3.html',
#	drs4 => 'http://www.drs.ch/import/webradio/playerinfos_infoszursendung_drs4.html',
};

my $icons = {
	drs1 => 'http://radiotime-logos.s3.amazonaws.com/s17465q.png',
	drs2 => 'http://radiotime-logos.s3.amazonaws.com/s18553q.png',
	drs3 => 'http://radiotime-logos.s3.amazonaws.com/s24862q.png',
};

# http://modules.drs.ch/data/attachments/DRS%20Stream%20URLs.pdf
#my $urlRegex = qr/skypro.*(drs[1-3]|regi_[a-z_]+|drs_event)/i;
my $urlRegex = qr/(?:skypro).*\/(drs[1-3])\//i;
my $urlParseRegex = qr/(?:srg-ssr|streamserver|skypro).*(drs[1-3]|regi_[a-z_]+|drs_event)/i;

sub getDisplayName {
	return 'PLUGIN_DRS_META';
}

sub initPlugin {
	my $class = shift;
	
	$VERSION = $class->_pluginDataFor('version');
	
#	Slim::Formats::RemoteMetadata->registerProvider(
#		match => $urlRegex,
#		func  => \&provider,
#	);

	Slim::Formats::RemoteMetadata->registerParser(
		match => $urlParseRegex,
		func  => \&parser,
	);
}

sub _pluginDataFor {
	my $class = shift;
	my $key   = shift;

	my $pluginData = Slim::Utils::PluginManager->dataForPlugin($class);

	if ($pluginData && ref($pluginData) && $pluginData->{$key}) {
		return $pluginData->{$key};
	}

	return undef;
}


my $meta = {
	drs1 => { busy => 0, title => 'DRS1', icon => $icons->{drs1}, cover => $icons->{drs1} },
	drs2 => { busy => 0, title => 'DRS2', icon => $icons->{drs2}, cover => $icons->{drs2}  },
	drs3 => { busy => 0, title => 'DRS3', icon => $icons->{drs3}, cover => $icons->{drs3}  },
};


sub parser {
	my ( $client, $url, $metadata ) = @_;
	
	if ($metadata =~ (/StreamTitle=\'(.*?)\'(?:;|$)/)) {
		my $titleInfo = $1;
		
		# what station?
		$url =~ $urlParseRegex;
		my $station = lc($1 || 'drs3');
		
		main::DEBUGLOG && $log->is_debug && $log->debug("Icy metadata received for $station: $titleInfo");

		my ($artist, $title) = split(' - ', $titleInfo);
		
		my $info = {
			icon   => $icons->{$station},
			cover  => $icons->{$station},
			artist => undef,
			title  => $titleInfo,
		};
		
		if ($title && $artist) {
			
			$info->{artist} = _lowercase($artist);
			$info->{title} = _lowercase($title);

			getArtwork( $client, $station, $info );
		}

		my $song = $client->controller()->songStreamController()->song();
		
		if ($song) {
			$song->pluginData( wmaMeta => $info );
	
			Slim::Control::Request::notifyFromArray( $client, [ 'newmetadata' ] );
		}

		return 1;
	}
	
	main::DEBUGLOG && $log->is_debug && $log->debug("No metadata set for $url");
	
	return 0;
}

=p1
sub provider {
	my ( $client, $url ) = @_;
	my $station;

	# what station?
	$url =~ $urlRegex;
	$station = lc($1);
	
	# Regionaljournale sind DRS1
	if ($station =~ /(?:regi_|drs_event)/i) {
		$station = 'drs1';
	}

	# don't query the meta data every time we're called
	if ( $client->isPlaying && (!$meta->{$station} || $meta->{$station}->{ttl} <= time) && !$meta->{$station}->{busy} ) {
		main::DEBUGLOG && $log->is_debug && $log->debug('Fetching data for station ' . $station);

		$meta->{$station}->{busy} = 1;

		my $http = Slim::Networking::SimpleAsyncHTTP->new(
			sub {
				$meta->{ $station } = parseContent($client, shift->content, $station);
			},
			sub {
				# in case of an error, don't try too hard...
				$meta->{ $station } = parseContent($client, '', $station);
			}
		);
		$http->get($urls->{$station});
	}
	
	return $meta->{$station};
}

sub parseContent {
	my ( $client, $content, $station ) = @_;

	my $info = {
		icon   => $icons->{$station},
		cover  => $icons->{$station},
		artist => undef,
		title  => undef,
		ttl    => time + cacheTTL,
	};
	
	# hopefully we get title/artist information
	if ($content =~ /"active_with_icon"[^>]*>(.*?)<\/span/sig) {
		$content = HTML::Entities::decode($1);
	}
	
	# if no song is available, try to get the current show's name 
	elsif ($content =~ /<b>(.*?)<\/b>.*class="inactive_with_icon"/sig) {
		$content = '';
		$info->{title} = HTML::Entities::decode($1);
	}
	
	# ...otherwise try to get the last played song
	elsif ($content =~ /"active"[^>]*>(.*?)<\/span/sig) {
		$content = HTML::Entities::decode($1);
	}
	
	# oh well...
	else {
		$content = '';
	}

	if ( $content ) {
		my $i = 0;
		map {
			$info->{ ('artist', 'title', '__playtime', '__dummy1', '__dummy2')[$i++] } = _lowercase($_);
		} split(/<br[\/]*>/i, $content);
	}
	
	$info->{title} ||= uc($station);

	main::DEBUGLOG && $log->is_debug && $log->debug("Got station information for $station - artist: $info->{artist}, title: $info->{title}");

	$info->{busy} = 0;

	if ( $info->{title} ne $meta->{$station}->{title} 
		|| $info->{artist} ne $meta->{$station}->{artist} ) {
	
		getArtwork( $client, $station, $info );
	}

	Slim::Control::Request::notifyFromArray( $client, [ 'playlist', 'newsong', $info->{title} ] );
		
	return $info;
}
=cut

sub getArtwork {
	my ( $client, $station, $info ) = @_;
	
	return if $station ne 'drs3';
	return unless $info->{artist};

	main::DEBUGLOG && $log->is_debug && $log->debug("Let's get some artwork...");		
	
	my $args = '?track=' . URI::Escape::uri_escape_utf8( $info->{title} || '' )
		. '&artist=' . URI::Escape::uri_escape_utf8( $info->{artist} || '' ); 

	my $http = Slim::Networking::SimpleAsyncHTTP->new(
		sub {
			my $html = shift;

			if ( $html && $html->content && $html->content =~ /^http/ ) {
				my $imageUrl = $html->content;
				
				main::DEBUGLOG && $log->is_debug && $log->debug("Found title artwork: " . $imageUrl);
				
				#$meta->{$station}->{icon} = $meta->{$station}->{cover} = $imageUrl;

				$client->playingSong->pluginData( wmaMeta => {
					icon   => $imageUrl,
					cover  => $imageUrl,
					artist => $info->{artist},
					title  => $info->{title},
				} );

				Slim::Control::Request::notifyFromArray( $client, [ 'newmetadata' ] );
			}
			elsif ( $html && $html->content ) {
				
				main::DEBUGLOG && $log->is_debug && $log->debug("Nothing found: " . $html->content);
				
			}
			else {
				
				main::DEBUGLOG && $log->is_debug && $log->debug(Data::Dump::dump($html));
				
			}
		},
		sub {
			logError(Data::Dump::dump(shift));
		},
	);
	
	$http->get( SEARCH_URL . $args );
	
	return 1;
}

sub _lowercase {
	my $s = shift;
	
	return $s if $s =~ /[a-z]+/;
	
	# uppercase all first characters
	$s =~ s/([\w'\`[:alpha:]äöüéàèçôî]+)/\u\L$1/ig;
		
	# fallback in case locale is wrong 
#	$s =~ tr/äöüàéèçôî/ÄÖÜÀÉÈÇÔÎ/;

	$s =~ s/^\s+//;
	$s =~ s/[\:\s]+$//;
	
	return $s;
}

1;